@@ -6,15 +6,15 @@ import types
 import re
 from functools import partial
 from wsgiref.headers import Headers as BaseHeaders
-from collections import Mapping, MutableSequence, Sequence
 import itertools
 import logging
 import inspect
 import copy
 from socket import gethostname
 
-from .compat.environ import *
-from .compat.imports import BaseHTTPRequestHandler, parse as urlparse, urlencode
+from datatypes import Url as BaseUrl, Host
+
+from .compat import *
 from .decorators.utils import _property
 from .utils import AcceptHeader, ByteString, MimeType, String, Base64, Deepcopy
 
@@ -171,10 +171,7 @@ class Headers(BaseHeaders, Mapping):
             self[k] = v
 
     def copy(self):
-        return self.__deepcopy__()
-
-    def __deepcopy__(self):
-        return type(self)(self._headers)
+        return Deepcopy().copy(self)
 
     def list(self):
         """Return all the headers as a list of headers instead of a dict"""
@@ -188,7 +185,7 @@ class Environ(Headers):
         return v
 
 
-class Url(String):
+class Url(BaseUrl):
     """a url object on steroids, this is here to make it easy to manipulate urls
     we try to map the supported fields to their urlparse equivalents, with some additions
 
@@ -207,328 +204,14 @@ class Url(String):
     .fragment = anchor
     .anchor = fragment
     .uri = /bar/che?baz=boom#anchor
-    .host(...) = httop://foo.com/...
-    .base(...) = httop://foo.com/bar/che/...
-    .controller(...) = httop://foo.com/bar/...
+    .host(...) = http://foo.com/...
+    .base(...) = http://foo.com/bar/che/...
+    .controller(...) = http://foo.com/bar/...
     """
-    scheme = "http"
-
-    username = None
-
-    password = None
-
-    hostname = ""
-
-    port = None
-
-    netloc = ""
-
-    path = ""
-
-    query_kwargs = {}
-
-    fragment = ""
-
     class_path = ""
 
     module_path = ""
 
-    @property
-    def root(self):
-        """just return scheme://netloc"""
-        return urlparse.urlunsplit((
-            self.scheme,
-            self.netloc,
-            "",
-            "",
-            ""
-        ))
-
-    @property
-    def anchor(self):
-        """alternative name for fragment"""
-        return self.fragment
-
-    @property
-    def uri(self):
-        """return the uri, which is everything but base (no scheme, host, etc)"""
-        uristring = self.path
-        if self.query:
-            uristring += "?{}".format(self.query)
-        if self.fragment:
-            uristring += "#{}".format(self.fragment)
-
-        return uristring
-
-    def __new__(cls, urlstring=None, **kwargs):
-        parts = cls.merge(urlstring, **kwargs)
-        urlstring = parts.pop("urlstring")
-        instance = super(Url, cls).__new__(cls, urlstring)
-        for k, v in parts.items():
-            setattr(instance, k, v)
-        return instance
-
-    @classmethod
-    def keys(cls):
-        # we need to ignore property objects also
-        is_valid = lambda k, v: not k.startswith("__") and not callable(v) and not isinstance(v, property)
-        keys = set(k for k, v in inspect.getmembers(cls) if is_valid(k, v))
-        return keys
-
-    @classmethod
-    def merge(cls, urlstring="", **kwargs):
-        # we handle port before any other because the port of host:port in hostname takes precedence
-        # the port on the host would take precedence because proxies mean that the
-        # host can be something:10000 and the port could be 9000 because 10000 is
-        # being proxied to 9000 on the machine, but we want to automatically account
-        # for things like that and then if custom behavior is needed then this method
-        # can be overridden
-        parts = {
-            "hostname": cls.hostname,
-            "port": cls.port,
-            "query_kwargs": dict(cls.query_kwargs),
-            "class_path": cls.class_path,
-            "module_path": cls.module_path,
-            "scheme": cls.scheme,
-            "netloc": cls.netloc,
-            "path": cls.path,
-            "fragment": cls.fragment,
-            "username": cls.username,
-            "password": cls.password,
-        }
-
-        if urlstring:
-            properties = [
-                "scheme", # 0
-                "netloc", # 1
-                "path", # 2
-                "fragment", # 5
-                "username",
-                "password",
-                "hostname",
-                "port",
-                "query", # 4
-            ]
-
-            if re.match(r"^\S+://", urlstring) or urlstring.startswith("//"):
-                o = urlparse.urlsplit(String(urlstring))
-
-            else:
-                # if we don't have a scheme we put // in front of it so it will
-                # still parse correctly
-                s = "//{}".format(String(urlstring))
-                o = urlparse.urlsplit(s)
-
-            for k in properties:
-                v = getattr(o, k)
-                if v:
-                    parts[k] = v
-
-            query = parts.get("query", "")
-            if query:
-                parts["query_kwargs"].update(cls.parse_query(query))
-
-        query = kwargs.pop("query", "")
-        if query:
-            parts["query_kwargs"].update(cls.parse_query(query))
-
-        query_kwargs = kwargs.pop("query_kwargs", {})
-        if query_kwargs:
-            parts["query_kwargs"].update(query_kwargs)
-
-        parts["query"] = ""
-        if parts["query_kwargs"]:
-            parts["query"] = cls.unparse_query(parts["query_kwargs"])
-
-        for k, v in kwargs.items():
-            parts[k] = v
-
-        ports = {
-            "http": 80,
-            "https": 443,
-        }
-        domain, port = cls.split_hostname_from_port(parts["hostname"])
-        parts["hostname"] = domain
-        if port:
-            parts["port"] = kwargs.get("port", port)
-
-        if not parts.get("port", None):
-            if "default_port" in kwargs:
-                parts["port"] = kwargs["default_port"]
-            else:
-                parts["port"] = ports.get(parts["scheme"], None)
-
-        # make sure port is an int
-        if parts["port"]:
-            parts["port"] = int(parts["port"])
-
-        hostloc = parts["hostname"]
-        port = parts["port"]
-        # we don't want common ports to be a part of a .geturl() call, but we do
-        # want .port to return them
-        if port and port not in set(ports.values()):
-            hostloc = '{}:{}'.format(hostloc, port)
-        parts["hostloc"] = hostloc
-
-        parts["netloc"] = parts["hostloc"]
-        username = kwargs.get("username", parts["username"])
-        password = kwargs.get("password", parts["password"])
-        if username:
-            parts["netloc"] = "{}:{}@{}".format(
-                username or "",
-                password or "",
-                parts["hostloc"]
-            )
-
-        parts["path"] = "/".join(cls.normalize_paths(parts["path"]))
-
-        parts["urlstring"] = urlparse.urlunsplit((
-            parts["scheme"],
-            parts["netloc"],
-            parts["path"],
-            parts["query"],
-            parts["fragment"],
-        ))
-
-        for k in parts:
-            if isinstance(parts[k], bytes):
-                parts[k] = String(parts[k])
-
-        return parts
-
-    @classmethod
-    def parse_query(cls, query):
-        """return name=val&name2=val2 strings into {name: val} dict"""
-        if not query: return {}
-
-        if isinstance(query, bytes):
-            query = String(query)
-
-        # https://docs.python.org/2/library/urlparse.html
-        query_kwargs = urlparse.parse_qs(query, True, strict_parsing=True)
-        return cls.normalize_query_kwargs(query_kwargs)
-
-    @classmethod
-    def normalize_query_kwargs(cls, query):
-        d = {}
-        # https://docs.python.org/2/library/urlparse.html
-        for k, kv in query.items():
-            #k = k.rstrip("[]") # strip out php type array designated variables
-            if isinstance(k, bytes):
-                k = String(k)
-
-            if len(kv) > 1:
-                d[k] = kv
-
-            else:
-                d[k] = kv[0]
-
-        return d
-
-    @classmethod
-    def unparse_query(cls, query_kwargs):
-        return urlencode(query_kwargs, doseq=True)
-
-    @classmethod
-    def normalize_paths(cls, *paths):
-        args = []
-        for ps in paths:
-            if isinstance(ps, basestring):
-                args.extend(filter(None, ps.split("/")))
-                #args.append(ps.strip("/"))
-            else:
-                for p in ps:
-                    args.extend(cls.normalize_paths(p))
-        return args
-
-    def _normalize_params(self, *paths, **query_kwargs):
-        """a lot of the helper methods are very similar, this handles their arguments"""
-        kwargs = {}
-
-        if paths:
-            fragment = paths[-1]
-            if fragment:
-                if fragment.startswith("#"):
-                    kwargs["fragment"] = fragment
-                    paths.pop(-1)
-
-            kwargs["path"] = "/".join(self.normalize_paths(*paths))
-
-        kwargs["query_kwargs"] = query_kwargs
-        return kwargs
-
-    @classmethod
-    def split_hostname_from_port(cls, hostname, default_port=None):
-        """given a hostname:port return a tuple (hostname, port)"""
-        bits = hostname.split(":", 2)
-        p = int(default_port) if default_port else default_port
-        d = bits[0]
-        if len(bits) == 2:
-            p = int(bits[1])
-
-        return d, p
-
-    def create(self, *args, **kwargs):
-        return type(self)(*args, **kwargs)
-
-    def add(self, **kwargs):
-        """Just a shortcut to change the current url, equivalent to Url(self, **kwargs)"""
-        if "path" in kwargs:
-            path = kwargs["path"]
-            if isinstance(path, bytes):
-                path = String(path)
-            if not path[0].startswith("/"):
-                paths = self.normalize_paths(self.path, path)
-            else:
-                paths = self.normalize_paths(path)
-            kwargs["path"] = "/".join(paths)
-        return self.create(self, **kwargs)
-
-    def subtract(self, *paths, **kwargs):
-        sub_kwargs = self.jsonable()
-
-        path2 = self.normalize_paths(paths)
-        path2.extend(self.normalize_paths(kwargs.pop("path", "")))
-        if path2:
-            sub_path = self.normalize_paths(self.path)
-            for p in path2:
-                try:
-                    sub_path.remove(p)
-                except ValueError:
-                    pass
-
-            sub_kwargs["path"] = sub_path
-
-        for k, v in kwargs.items():
-            if k == "query_kwargs":
-                for qk, qv in kwargs[k].items():
-                    if str(sub_kwargs[k][qk]) == str(qv):
-                        sub_kwargs[k].pop(qk)
-
-            else:
-                if str(sub_kwargs[k]) == str(v):
-                    sub_kwargs.pop(k)
-
-        return self.create(**sub_kwargs)
-
-    def parent(self, *paths, **query_kwargs):
-        """create a new Url instance one level up from the current Url instance
-
-        so if self contains /foo/bar then self.parent() would return /foo
-
-        :param *paths: list, the paths to append to the parent path
-        :param **query_kwargs: dict, any query string params to add
-        :returns: new Url instance
-        """
-        kwargs = self._normalize_params(*paths, **query_kwargs)
-        path_args = self.path.split("/")
-        if path_args:
-            urlstring = self.subtract(path_args[-1])
-        else:
-            urlstring = self
-
-        return urlstring.add(**kwargs)
-
     def module(self, *paths, **query_kwargs):
         """create a new Url instance using the module path as a base
 
@@ -553,7 +236,7 @@ class Url(String):
         appended to `/foo/bar`
 
         :example:
-            # controller foo.BarController
+            # controller foo.Bar(Controller)
 
             print url # http://host.com/foo/bar/some_random_path
 
@@ -573,199 +256,6 @@ class Url(String):
                 kwargs["path"] = self.class_path
         return self.create(self.root, **kwargs)
 
-    def base(self, *paths, **query_kwargs):
-        """create a new url object using the current base path as a base
-
-        if you had requested /foo/bar, then this would append *paths and **query_kwargs
-        to /foo/bar
-
-        :example:
-            # current path: /foo/bar
-
-            print url # http://host.com/foo/bar
-
-            print url.base() # http://host.com/foo/bar
-            print url.base("che", boom="bam") # http://host/foo/bar/che?boom=bam
-
-        :param *paths: list, the paths to append to the current path without query params
-        :param **query_kwargs: dict, any query string params to add
-        """
-        kwargs = self._normalize_params(*paths, **query_kwargs)
-        if self.path:
-            if "path" in kwargs:
-                paths = self.normalize_paths(self.path, kwargs["path"])
-                kwargs["path"] = "/".join(paths)
-            else:
-                kwargs["path"] = self.path
-        return self.create(self.root, **kwargs)
-
-    def host(self, *paths, **query_kwargs):
-        """create a new url object using the host as a base
-
-        if you had requested http://host/foo/bar, then this would append *paths and **query_kwargs
-        to http://host
-
-        :example:
-            # current url: http://host/foo/bar
-
-            print url # http://host.com/foo/bar
-
-            print url.host() # http://host.com/
-            print url.host("che", boom="bam") # http://host/che?boom=bam
-
-        :param *paths: list, the paths to append to the current path without query params
-        :param **query_kwargs: dict, any query string params to add
-        """
-        kwargs = self._normalize_params(*paths, **query_kwargs)
-        return self.create(self.root, **kwargs)
-
-    def copy(self):
-        return self.__deepcopy__()
-
-    def __copy__(self):
-        return self.__deepcopy__()
-
-    def __deepcopy__(self, memodict={}):
-        return self.create(
-            scheme=self.scheme,
-            username=self.username,
-            password=self.password,
-            hostname=self.hostname,
-            port=self.port,
-            path=self.path,
-            query_kwargs=self.query_kwargs,
-            fragment=self.fragment,
-            class_path=self.class_path,
-            module_path=self.module_path,
-        )
-
-    def __add__(self, other):
-        ret = ""
-        if isinstance(other, Mapping):
-            ret = self.add(query_kwargs=other)
-
-        elif isinstance(other, MutableSequence):
-            ret = self.add(path=other)
-
-        elif isinstance(other, basestring):
-            ret = self.add(path=other)
-
-        elif isinstance(other, Sequence):
-            ret = self.add(path=other)
-
-        else:
-            raise ValueError("Not sure how to add {}".format(type(other)))
-
-        return ret
-    __iadd__ = __add__
-
-    def __truediv__(self, other):
-        ret = ""
-        if isinstance(other, MutableSequence):
-            ret = self.add(path=other)
-
-        elif isinstance(other, basestring):
-            ret = self.add(path=other)
-
-        elif isinstance(other, Sequence):
-            ret = self.add(path=other)
-
-        else:
-            raise ValueError("Not sure how to add {}".format(type(other)))
-
-        return ret
-    __itruediv__ = __truediv__
-
-    def __sub__(self, other):
-        """Return a new url with other removed"""
-        ret = ""
-        if isinstance(other, Mapping):
-            ret = self.subtract(query_kwargs=other)
-
-        elif isinstance(other, MutableSequence):
-            ret = self.subtract(path=other)
-
-        elif isinstance(other, basestring):
-            ret = self.subtract(path=other)
-
-        elif isinstance(other, Sequence):
-            ret = self.subtract(path=other)
-
-        else:
-            raise ValueError("Not sure how to add {}".format(type(other)))
-
-        return ret
-    __isub__ = __sub__
-
-    def jsonable(self):
-        ret = {}
-        for k in self.keys():
-            v = getattr(self, k)
-            if k == "query_kwargs":
-                ret[k] = dict(v)
-            else:
-                ret[k] = v
-
-        return ret
-
-
-class Host(tuple):
-    """Creates a tuple (hostname, port) that can be passed to built-in PYthon server
-    classes and anything that uses that same interface, does all the lifting to
-    figure out what the hostname and port should be based on the passed in input,
-    so you can pass things like hostname:port, or (host, port), etc.
-    """
-    @property
-    def hostname(self):
-        """Return just the hostname with no port or scheme or anything
-
-        :returns: string
-        """
-        return self[0]
-
-    @property
-    def port(self):
-        """return just the port
-
-        :returns: int
-        """
-        return self[1]
-
-    @property
-    def hostloc(self):
-        """Returns hostname:port
-
-        :returns: string
-        """
-        return "{}:{}".format(self.hostname, self.port)
-    netloc = hostloc
-
-    def __new__(cls, host, port=None):
-        u = Url(host, default_port=port)
-        return super(Host, cls).__new__(cls, [u.hostname, u.port if u.port else 0])
-
-    def __str__(self):
-        return self.__bytes__() if is_py2 else self.__unicode__()
-
-    def __unicode__(self):
-        return String(self.hostloc)
-
-    def __bytes__(self):
-        return ByteString(self.hostloc)
-
-    def client(self):
-        """Url can technically hold a hostname like 0.0.0.0, this will compensate
-        for that, useful for test clients
-
-        :returns: a netloc (host:port) that a client can use to make a request
-        """
-        netloc = ""
-        domain, port = self
-        netloc = gethostname() if domain == "0.0.0.0" else domain
-        if port:
-            netloc += ":{}".format(port)
-        return netloc
-
 
 class Http(object):
     def __init__(self):
@@ -834,14 +324,12 @@ class Http(object):
         return copy.deepcopy(self)
 
     def __deepcopy__(self, memodict=None):
-        if not memodict:
-            memodict = {}
+        memodict = memodict or {}
 
         if self.controller_info:
             memodict.setdefault("controller_info", self.controller_info)
 
-        instance = type(self)()
-        return Deepcopy.copy(self, memodict, instance)
+        return Deepcopy().copy(self, memodict)
 
     def is_json(self):
         ct = self.get_header('Content-Type')
@@ -1218,6 +706,7 @@ class Response(Http):
     @code.setter
     def code(self, v):
         self._code = v
+        self._status = None
 
     @property
     def status_code(self): return self.code
